#ifndef MONSTERS_H
#define MONSTERS_H

#include <QString>
#include <QImage>
#include <vector>
#include "../common/sources/Map.h"
#include "timer.h"

//---------------------------------------------------------------------------------------------
enum EMonsterPos
{
    eMonsterPosNone,    // empty monster
    eMonsterPosNorthWest,
    eMonsterPosNorthEast,
    eMonsterPosSouthWest,
    eMonsterPosSouthEast,
    eMonsterPosCenter
};

enum EMonsterDir
{
    eMonsterDirUp,
    eMonsterDirDown,
    eMonsterDirLeft,
    eMonsterDirRight
};

enum EMonsterState
{
    eMonsterState_Wandering,
    eMonsterState_Fight
};

#define MONSTER_TYPE_SCREAMER   7

class CMonsterGroup2;

//---------------------------------------------------------------------------------------------
class CMonster
{
public:
    void    update(CMonsterGroup2& group, CVec2 mapPos, uint8_t type, bool isInFront);
    void    wander();
    void    fight(CVec2 mapPos, uint8_t type, bool isInFront);
    void    initNextAttack(uint8_t type);
    uint8_t chooseTarget(CVec2 mapPos);
    void    load(FILE* handle);
    void    save(FILE* handle);

    EMonsterPos pos;
    EMonsterDir dir;
    int life;
    CTimer  displayAttackTimer;
    CTimer  nextAttackTimer;
};

//---------------------------------------------------------------------------------------------
class CMonsterGroup2
{
public:
    CMonsterGroup2();
    void    update(CVec2 mapPos, uint8_t type);
    void    wander(CVec2 mapPos, uint8_t type);
    void    fight(CVec2 mapPos, uint8_t type);
    void    hitMonster(CVec2 groupPos, int monsterNum, int damages);
    void    load(FILE* handle);
    void    save(FILE* handle);
    void    enterWanderState(int type);

    uint8_t getMonsterRow(CVec2 mapPos, EMonsterPos monsterPos);
    bool    isMonsterInFront(CVec2 mapPos, EMonsterPos monsterPos);

    CMonster        mMonsters[4];
    EMonsterDir     dir;
    EMonsterState   state;
    CVec2           startPos;
    CTimer          moveTimer;

private:
    CVec2   getNewPos(CVec2 pos, EWallSide side);
    bool    canMove(CVec2 pos, EWallSide side);
    void    initMoveTimer(int type);
};

//---------------------------------------------------------------------------------------------
class CMonsters
{
public:
    struct SMonsterData
    {
        QString frontImage;
        QString sideImage;
        QString backImage;
        QString attackImage;
        QString attackSound;
        QString moveSound;
        int     attackDisplayTime;
        int     timeBetweenAttack;
        int     movementTime;
        int     defense;
        int     baseHealth;
        int     attack;
        int     dexterity;
    };

    CMonsters();
    void    readMonstersDB();
    void    init();
    void    draw(QImage* image, CVec2 mapPos, CVec2 tablePos, int row);
    void    update();
    CVec2   getMonsterPos2(CVec2 mapPos, CVec2 tablePos, EMonsterPos monsterPos);
    static CVec2   getMonsterPos(float x, float y);

    std::vector<SMonsterData>   monstersDatas;
    std::vector<CMonsterGroup2> monsterGroups;

private:
    int     getStartingLife(int type);
};

extern CMonsters    monsters;
extern const float monsterPosDir[4][5][2];

#endif // MONSTERS_H
